/////////////////////////////////////////////////////////////////////////
// cmdenabl.cpp    Source for cmdenabl.exe sample application
//                 demonstrating several different techniques of
//                 enabling menu items functioning in harmony using
//                 OWL 2.x and above.
//
// Written by :    George Cross, Borland C++ Developer Support
// Date       :    June 11, 1996
// Compiler   :    Borland C++ 5.0, but is valid for 4.x and above
/////////////////////////////////////////////////////////////////////////

#include <owl/pch.h>
#include "cmdenabl.h"
#include "cmdenabl.rh"

/////////////////////////////////////////////////////////////////////////
// Frame window class
/////////////////////////////////////////////////////////////////////////
DEFINE_RESPONSE_TABLE1(TCmdEnableFrame, TFrameWindow)
  EV_WM_INITMENUPOPUP,
END_RESPONSE_TABLE;


//EvInitMenuPopup() can be used in the OWL frame window to
//do the job of command enablers if you really want.
//
void TCmdEnableFrame::EvInitMenuPopup(HMENU hPopupMenu,
                                       uint index, bool sysMenu)
{
  //Call base class function otherwise the client window's
  //OWL command enablers won't be called
  //
  TFrameWindow::EvInitMenuPopup(hPopupMenu, index, sysMenu);

  //Now enable the 'Run' popup menu item depending on our
  //global variable 'TCmdEnableApp->running'
  //
  if (hPopupMenu == TMenu(HWindow).GetSubMenu(0)){
    if (dynamic_cast<TCmdEnableApp*>(GetApplication())->running == true)
        TMenu(hPopupMenu).EnableMenuItem(CM_RUN, MF_GRAYED);
    else
        TMenu(hPopupMenu).EnableMenuItem(CM_RUN, MF_ENABLED);
  }
}


/////////////////////////////////////////////////////////////////////////
// Client window class
/////////////////////////////////////////////////////////////////////////
DEFINE_RESPONSE_TABLE1(TCmdEnableWindow, TWindow)
  EV_COMMAND(CM_RUN,CmRun),
  EV_COMMAND(CM_DUPLICATEOFRUN, CmRun),
  EV_COMMAND(CM_WHILERUNNING, CmWhileRunning),
  EV_COMMAND_ENABLE(CM_WHILERUNNING, CeWhileRunning),
END_RESPONSE_TABLE;


//When CmRun() function is called, the 'Duplicate of Run' top level
//menu item is disabled and a loop is entered.  When the loop is
//done, the 'Duplicate of Run' menu item is re-enabled.
//
//This is the only mechanism I found to disable top level menu
//items during some processing.  Note the call to DrawMenuBar()
//and PumpWaitingMessages().  Try commenting each out in turn
//and notice the changes in the application's behaviour.
//
void TCmdEnableWindow::CmRun()
{
  dynamic_cast<TCmdEnableApp*>(GetApplication())->running = true;
  MessageBeep(-1);

  //Disable top level menu item
  TMenu m(Parent->HWindow);
  m.EnableMenuItem(CM_DUPLICATEOFRUN, MF_GRAYED);
  Parent->DrawMenuBar();

  //Run a loop to simulate some processing.  Call PumpWaitingMessages
  //so that the user can select the menu and do other things in duration
  uint32 start = GetTickCount();
  while(GetTickCount() - start < 2500)
     GetApplication()->PumpWaitingMessages();

  //Re-enable top level menu item
  MessageBeep(-1);
  m.EnableMenuItem(CM_DUPLICATEOFRUN, MF_ENABLED);
  Parent->DrawMenuBar();
  dynamic_cast<TCmdEnableApp*>(GetApplication())->running = false;
}


//Just beep
//
void TCmdEnableWindow::CmWhileRunning()
{
  MessageBeep(-1);
}

//Enables the Enabled While Running popup menu item depending
//on our global variable 'TCmdEnableApp->running'
//
void TCmdEnableWindow::CeWhileRunning(TCommandEnabler& ce){
    ce.Enable(dynamic_cast<TCmdEnableApp*>(GetApplication())->running);
}


/////////////////////////////////////////////////////////////////////////
// Application object
/////////////////////////////////////////////////////////////////////////
void TCmdEnableApp::InitMainWindow()
{
  // Construct the frame window
  TCmdEnableFrame* frame = new TCmdEnableFrame(0,
                                         "Command-enabling sample app",
                                         new TCmdEnableWindow, true);

  // Set the main window and its menu
  SetMainWindow(frame);
  GetMainWindow()->AssignMenu("COMMANDS");
}


/////////////////////////////////////////////////////////////////////////
// Program entry point
/////////////////////////////////////////////////////////////////////////
int OwlMain(int /*argc*/, char* /*argv*/ [])
{
  return TCmdEnableApp().Run();
}

